//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*  MonsterMoto Shield Example Sketch
  date: 5/24/11  code by: Jim Lindblom  hardware by: Nate Bernstein  SparkFun Electronics
  VNH2SP30 high-current full-bridge motor drivers.
/*/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define CS_THRESHOLD 100

//  SN754410 pin definitions
/*
Clockwise          INA: 1   INB: 0                  - Direction Digital 
Counter-clockwise  INA: 0   INB: 1  
EN -> GND
66 -  pin 9 int inApin = 9 ;  // INA: Clockwise input - Counter-clockwise input  
66 -  pin 10 int inBpin = 10 ; // INB: 
      pin 11  - Speed (PWM)
int cspin = A0 ; // CS: Current sense ANALOG input            
int enpin = 12 ; // EN: 
*/
int cspin = A0 ; // CS: Current sense ANALOG input 
int current = 0;
// int enpin = 10 ; // EN1 754410 solenoid standard output
int enpin2 = 15 ; // EN2 754410 motor output A1
//int statpin = 13; //  LED
// int pwmpin = 9 ; // PWM input
   

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DCC Accessory / Function Decoder
// Author: Ruud Boer - September 2015
// This sketch turns an Arduino into a DCC decoder with max 17 function outputs.
// Output pins used: 3-19 (14-19 = A0-A5). Pin becomes LOW when accessory is switched ON
// Modes: 1-continuous, 2=oneshot, 3=flasher with 2 alternatin outputs, 4=signal with 2 inverted outputs
// The DCC signal is fed to pin 2 (=Interrupt 0).
// Optocoupler schematics for DCC to 5V conversion: www.rudysmodelrailway.wordpress.com/software
// Many thanks to www.mynabay.com for publishing their DCC monitor and -decoder code.
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// IMPORTANT: GOTO lines 46 and 70 to configure some data!
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <DCC_Decoder.h>
#define kDCC_INTERRUPT 0

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Fill in the number of accessories / functions you want to control
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
const byte maxaccessories = 7;
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef struct {
  int               address;          // User Configurable. DCC address to respond to
  byte              mode;             // User Configurable. Mode: 1=Continuous, 2=Oneshot, 3=Flasher
  byte              outputPin;        // User Configurable. Arduino pin where accessory is connected to
  byte              outputPin2;       // User Configurable. 2nd pin for AlternatingFlasher (e.g. railway crossing)
  int               ontime;           // User Configurable. Oneshot or Flasher on time in ms
  int               offtime;          // User Configurable. Flasher off time in ms
  byte              onoff;            // User Configurable. Initial state of accessory output: 1=on, 0=off (ON = pin LOW)
  byte              onoff2;           // User Configurable. Initial  state of 2nd output: 1=on, 0=off
  byte              dccstate;         // Internal use. DCC state of accessory: 1=on, 0=off
  byte              finished;         // Internal use. Memory that says the Oneshot is finished
  unsigned long     onMilli;          // Internal use.
  unsigned long     offMilli;         // Internal use.
} 
DCCAccessoryAddress;
DCCAccessoryAddress accessory[maxaccessories];

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Fill in the attributes for every accessory / function
// COPY - PASTE as many times as you have functions. The amount must be same as in line 18 above!
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void ConfigureDecoderFunctions() // The amount of accessories must be same as in line 26 above!
{

  accessory[0].address = 66; // DCC address TurnTable Motor rotation way
  accessory[0].mode = 4; // Continuous: HIGH until DCC switches the address off again
  accessory[0].outputPin = 11; // CW
  accessory[0].outputPin2 = 12; // CCW
  accessory[0].onoff2 = 1; // Initially set Red signal to ON
  
  accessory[1].address = 67; // DCC address TurnTable Solenoid allow Motor rotation
  accessory[1].mode = 2; // Oneshot: HIGH for ontime ms, then LOW and stays LOW.
  accessory[1].outputPin = 10;
  accessory[1].ontime = 500;
 
  accessory[2].address = 34; 
  accessory[2].mode = 4; // 4 Continuous: HIGH until DCC switches the address off again
  accessory[2].outputPin = 3; // //  Green Signal
  accessory[2].outputPin2 =  4; //  Red signal
  accessory[2].onoff2 = 1; // Initially set Red signal to ON
    
  accessory[3].address = 33; // DCC address flash light
  accessory[3].mode = 3; // 3 Flasher: HIGH for ontime ms, LOW for offtime ms, repeats till DCC off
  accessory[3].outputPin = 5; //
  accessory[3].outputPin2 = 8; // Flasher can use 2 outputs, they will flash  on/off alternatively
  accessory[3].ontime = 500;
  accessory[3].offtime = 500;
  accessory[3].onoff = 1; // Always initially signal set to ON
  accessory[3].onoff2 = 0; // Always initially signal set to ON
  
  accessory[4].address = 37; // DCC address  
  accessory[4].mode = 4; // Continuous: HIGH until DCC switches the address off again
  accessory[4].outputPin = 14; // Green signal
  accessory[4].outputPin2 = 15; // Red Signal
  accessory[4].onoff2 = 1; // Initially set Red signal to ON

  accessory[5].address = 35; // DCC address  
  accessory[5].mode = 4; // Continuous: HIGH until DCC switches the address off again
  accessory[5].outputPin = 6; // Green signal
  accessory[5].outputPin2 = 7; // Red Signal
  accessory[5].onoff2 = 1; // Initially set Red signal to ON

  accessory[6].address = 36; // DCC address  Level Crossing
  accessory[6].mode = 4; // Continuous: HIGH until DCC switches the address off again
  accessory[6].outputPin = 9; // Green signal
  accessory[6].outputPin2 = 13; // Red Signal
  accessory[6].onoff2 = 1; // Initially set Red signal to ON

}  // END ConfigureDecoderFunctions

  //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  // DCC accessory packet handler 
  //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  void BasicAccDecoderPacket_Handler(int address, boolean activate, byte data)
  {
    // Convert NMRA packet address format to human address
    address -= 1;
    address *= 4;
    address += 1;
    address += (data & 0x06) >> 1;

    boolean enable = (data & 0x01) ? 1 : 0;

    for (int i=0; i<maxaccessories; i++)
    {
      if (address == accessory[i].address)
      {
        if (enable) accessory[i].dccstate = 1;
        else accessory[i].dccstate = 0;
      }
    }
  } //END BasicAccDecoderPacket_Handler

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Setup (run once)
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void setup() 
{ 
      
  ConfigureDecoderFunctions();
  DCC.SetBasicAccessoryDecoderPacketHandler(BasicAccDecoderPacket_Handler, true);
  DCC.SetupDecoder( 0x00, 0x00, kDCC_INTERRUPT );
  pinMode(2,INPUT_PULLUP); // Interrupt 0 with internal pull up resistor (can get rid of external 10k)

  for(int i=3; i<20; i++)
  {
    pinMode(i, OUTPUT);
    digitalWrite(i, LOW); //all function outputs are set to 0 at startup
  }
    digitalWrite(5, HIGH); // flasher set to 0 at startup
    digitalWrite(8, HIGH); // flasher set to 0 at startup

  digitalWrite(enpin2, HIGH); // EN2 754410 motor output A1
  pinMode(cspin, INPUT); // A0
  current = analogRead(cspin); // 

  Serial.begin (115200);
  Serial.println ("Decoder Plaque 07 adresses 66, 67 et 33 -> 36 ");
  Serial.println ("12-08-2022 ");
  Serial.println (current);
  delay(1000);
  
} // END setup
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Main loop (run continuous)
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void loop()
{
  static int addr = 0;

  DCC.loop(); // Loop DCC library
    
  if( ++addr >= maxaccessories ) addr = 0; // Next address to test

  if (accessory[addr].dccstate)
  {
    switch (accessory[addr].mode)
    {
    case 1: // Continuous
      accessory[addr].onoff = 1;
      break;
    case 2: // Oneshot
      if (!accessory[addr].onoff && !accessory[addr].finished)
      {
        accessory[addr].onoff = 1;
        accessory[addr].offMilli = millis() + accessory[addr].ontime;
      }
      if (accessory[addr].onoff && millis() > accessory[addr].offMilli)
      {
        accessory[addr].onoff = 0;
        accessory[addr].finished = true; //this is reset to flase below in the 'else' statement
      }
      break;
    case 3: // Flasher, is always an 'alternating' flasher together with .outputPin2
      if (accessory[addr].onoff && millis() > accessory[addr].offMilli)
      {
        accessory[addr].onoff = 0;
        accessory[addr].onoff2 = 1;
        accessory[addr].onMilli = millis() + accessory[addr].offtime;
      }
      if (!accessory[addr].onoff && millis() > accessory[addr].onMilli)
      {
        accessory[addr].onoff = 1;
        accessory[addr].onoff2 = 0;
        accessory[addr].offMilli = millis() + accessory[addr].ontime;
      }
      break;
    case 4: // Signal
      accessory[addr].onoff = 1;
      accessory[addr].onoff2 = 0;
      break;
    }
  }
  else //accessory[addr].dccstate == 0
  {
    accessory[addr].onoff = 0;
    if (accessory[addr].mode == 4) accessory[addr].onoff2 = 1; else accessory[addr].onoff2 = 0;
    if (accessory[addr].mode == 2) accessory[addr].finished = false; // Oneshot finished by DCCstate, not by ontime
  }

  // activate outputpin, based on value of onoff
  if (accessory[addr].onoff) {    digitalWrite(accessory[addr].outputPin, HIGH); }
  else digitalWrite(accessory[addr].outputPin, LOW);
  if (accessory[addr].onoff2) digitalWrite(accessory[addr].outputPin2, HIGH);
  else digitalWrite(accessory[addr].outputPin2, LOW);

} //END loop
